/*

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 */

#include "ui_local.h"

static menuframework_s s_performanceadv_menu;

static void PortalMapsMaxTexSizeControl( menucommon_t *unused )
{
	menucommon_t *menuitem = UI_MenuItemByName( "m_performanceadv_portalmaps_maxtexsize_slider" );
	menuitem->disabled = !menuitem->disabled;
}

static void ShadowsControl( menucommon_t *s )
{
	menucommon_t *menuitem, *slider;
	
	menuitem = UI_MenuItemByName( "m_performanceadv_shadowmap_maxtexsize" );
	menuitem->disabled = s->curvalue != 3;

	slider = UI_MenuItemByName( "m_performanceadv_shadowmap_maxtexsize_slider" );
	if( s->curvalue != 3 || !menuitem->curvalue )
		slider->disabled = qtrue;
	else
		slider->disabled = s->curvalue != 3;
}

static void ShadowMapMaxTexSizeControl( menucommon_t *unused )
{
	menucommon_t *menuitem = UI_MenuItemByName( "m_performanceadv_shadowmap_maxtexsize_slider" );
	menuitem->disabled = !menuitem->disabled;
}

static void ApplyButton( menucommon_t *unused )
{
	menucommon_t *menuitem;
	int maxfps;

	maxfps = atoi( UI_GetMenuitemFieldBuffer( UI_MenuItemByName( "m_performanceadv_maxfps" ) ) );
	trap_Cvar_SetValue( "cl_maxfps", (maxfps ? max( maxfps, 24 ) : 0) );

	menuitem = UI_MenuItemByName( "m_performanceadv_sleep" );
	trap_Cvar_SetValue( "cl_sleep", menuitem->curvalue );

	menuitem = UI_MenuItemByName( "m_performanceadv_filter" );
	if( menuitem->curvalue >= 2 )
	{
		trap_Cvar_Set( "r_texturemode", "GL_LINEAR_MIPMAP_LINEAR" );
		trap_Cvar_SetValue( "gl_ext_texture_filter_anisotropic", (1<<(menuitem->curvalue-1) ));
	}
	else
	{
		trap_Cvar_Set( "r_texturemode", menuitem->curvalue ? "GL_LINEAR_MIPMAP_LINEAR" : "GL_LINEAR_MIPMAP_NEAREST" );
		trap_Cvar_SetValue( "gl_ext_texture_filter_anisotropic", 0 );
	}

	menuitem = UI_MenuItemByName( "m_performanceadv_portalmaps" );
	trap_Cvar_SetValue( "r_portalmaps", menuitem->curvalue );

	menuitem = UI_MenuItemByName( "m_performanceadv_fastsky" );
	trap_Cvar_SetValue( "r_fastsky", menuitem->curvalue );

	menuitem = UI_MenuItemByName( "m_performanceadv_portalmaps_maxtexsize" );
	if( !menuitem->curvalue )
		trap_Cvar_SetValue( "r_portalmaps_maxtexsize", 0 );
	else
	{
		menuitem = UI_MenuItemByName( "m_performanceadv_portalmaps_maxtexsize_slider" );
		trap_Cvar_SetValue( "r_portalmaps_maxtexsize", menuitem->curvalue );
	}

#ifdef CGAMEGETLIGHTORIGIN
	menuitem = UI_MenuItemByName( "m_performanceadv_shadows" );
	trap_Cvar_SetValue( "cg_shadows", menuitem->curvalue );
	trap_Cvar_SetValue( "r_stencilbits", ( menuitem->curvalue > 1 )*8 ); //  2 is SHADOWS_STENCIL

	menuitem = UI_MenuItemByName( "m_performanceadv_shadowmap_maxtexsize" );
	if( !menuitem->curvalue )
		trap_Cvar_SetValue( "r_shadows_maxtexsize", 0 );
	else
	{
		menuitem = UI_MenuItemByName( "m_performanceadv_shadowmap_maxtexsize_slider" );
		trap_Cvar_SetValue( "r_shadows_maxtexsize", menuitem->curvalue );
	}
#else
	menuitem = UI_MenuItemByName( "m_video_cg_shadows" );
	trap_Cvar_SetValue( "r_shadows", menuitem->curvalue );
	trap_Cvar_SetValue( "r_stencilbits", ( menuitem->curvalue != 0 ) * 8 ); //  2 is SHADOWS_STENCIL
#endif

	menuitem = UI_MenuItemByName( "m_performanceadv_shadowmapsPCF" );
	trap_Cvar_SetValue( "r_shadows_pcf", (menuitem->curvalue ? (menuitem->curvalue+1) : 0) );

	menuitem = UI_MenuItemByName( "m_performanceadv_outlineworld" );
	trap_Cvar_SetValue( "cg_outlineWorld", menuitem->curvalue );

	menuitem = UI_MenuItemByName( "m_performanceadv_outlinemodels" );
	trap_Cvar_SetValue( "cg_outlineModels", menuitem->curvalue );

#if 0
	menuitem = UI_MenuItemByName( "m_performanceadv_glsl" );
	trap_Cvar_SetValue( "gl_ext_GLSL", menuitem->curvalue );
#endif

	menuitem = UI_MenuItemByName( "m_performanceadv_deluxemapping" );
	trap_Cvar_SetValue( "r_lighting_deluxemapping", menuitem->curvalue );

	menuitem = UI_MenuItemByName( "m_performanceadv_dynamiclight" );
	trap_Cvar_SetValue( "r_dynamiclight", menuitem->curvalue );

	menuitem = UI_MenuItemByName( "m_performanceadv_subdivisions" );
	trap_Cvar_SetValue( "r_subdivisions", (1<<(4 - menuitem->curvalue)) );

	menuitem = UI_MenuItemByName( "m_performanceadv_offsetmapping" );
	trap_Cvar_SetValue( "r_offsetmapping", menuitem->curvalue );

	menuitem = UI_MenuItemByName( "m_performanceadv_bloom" );
	trap_Cvar_SetValue( "r_bloom", menuitem->curvalue );

	trap_Cmd_ExecuteText( EXEC_APPEND, "vid_restart\n" );
}

#define NEARESTEXPOF2(x)  ((int)floor( ( log( max( (x), 1 ) ) - log( 1.5 ) ) / log( 2 ) + 1 ))

static void PerformanceAdv_Init( void )
{
	menucommon_t *menuitem;
	int yoffset = 0;

	static char *filter_names[] = { "bilinear", "trilinear", "2xAF", "4xAF", "8xAF", "16xAF", 0 };
	int anisotropic, spinindex;

	static char *maxtexsize_names[] = { "best", "custom", 0 };
	int maxtexsize = min( trap_Cvar_Value( "gl_max_texture_size" ), min( uis.vidWidth, uis.vidHeight ) );

	static char *dynamiclights_names[] = { "off", "best", "fast", 0 };

	static char *offsetmapping_names[] = { "off", "world", "decals", "world and decals", 0 };

#ifdef CGAMEGETLIGHTORIGIN
	static char *shadows_names[] = { "off", "simple", "stencil", "shadow maps", 0 };
#endif

	menuitem = UI_InitMenuItem( "m_performanceadv_title1", "ADVANCED", 0, yoffset, MTYPE_SEPARATOR, ALIGN_CENTER_TOP, uis.fontSystemBig, NULL );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	yoffset += trap_SCR_strHeight( menuitem->font );
	yoffset += trap_SCR_strHeight( menuitem->font );

	menuitem = UI_InitMenuItem( "m_performanceadv_maxfps", "max fps", 0, yoffset, MTYPE_FIELD, ALIGN_RIGHT_TOP, uis.fontSystemSmall, NULL );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	UI_SetupField( menuitem, trap_Cvar_String( "cl_maxfps" ), 4, -1 );
	yoffset += trap_SCR_strHeight( menuitem->font );

	menuitem = UI_InitMenuItem( "m_performanceadv_sleep", "sleep state between frames", 0, yoffset, MTYPE_SPINCONTROL, ALIGN_RIGHT_TOP, uis.fontSystemSmall, NULL );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	UI_SetupSpinControl( menuitem, offon_names, trap_Cvar_Value( "cl_sleep" ) );
	yoffset += trap_SCR_strHeight( menuitem->font );

	menuitem = UI_InitMenuItem( "m_performanceadv_filter", "texture filter", 0, yoffset, MTYPE_SPINCONTROL, ALIGN_RIGHT_TOP, uis.fontSystemSmall, NULL );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	anisotropic = trap_Cvar_Value( "gl_ext_texture_filter_anisotropic" );
	if( anisotropic >= 2 )
		spinindex = NEARESTEXPOF2( anisotropic ) + 1;
	else if( !Q_stricmp( trap_Cvar_String( "r_texturemode" ), "GL_LINEAR_MIPMAP_NEAREST" ) )
		spinindex = 0;
	else
		spinindex = 1;
	UI_SetupSpinControl( menuitem, filter_names, spinindex );
	yoffset += trap_SCR_strHeight( menuitem->font );

	menuitem = UI_InitMenuItem( "m_performanceadv_portalmaps", "portalmaps", 0, yoffset, MTYPE_SPINCONTROL, ALIGN_RIGHT_TOP, uis.fontSystemSmall, NULL );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	UI_SetupSpinControl( menuitem, offon_names, trap_Cvar_Value( "r_portalmaps" ) );
	yoffset += trap_SCR_strHeight( menuitem->font );

	menuitem = UI_InitMenuItem( "m_performanceadv_fastsky", "fastsky", 0, yoffset, MTYPE_SPINCONTROL, ALIGN_RIGHT_TOP, uis.fontSystemSmall, NULL );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	UI_SetupSpinControl( menuitem, offon_names, trap_Cvar_Value( "r_fastsky" ) );
	yoffset += trap_SCR_strHeight( menuitem->font );

	menuitem = UI_InitMenuItem( "m_performanceadv_portalmaps_maxtexsize", "portalmaps texture size", 0, yoffset, MTYPE_SPINCONTROL, ALIGN_RIGHT_TOP, uis.fontSystemSmall, PortalMapsMaxTexSizeControl );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	UI_SetupSpinControl( menuitem, maxtexsize_names, trap_Cvar_Value( "r_portalmaps_maxtexsize" ) );
	yoffset += trap_SCR_strHeight( menuitem->font );
	menuitem = UI_InitMenuItem( "m_performanceadv_portalmaps_maxtexsize_slider", "", 0, yoffset, MTYPE_SLIDER, ALIGN_RIGHT_TOP, uis.fontSystemSmall, NULL );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	UI_SetupSlider( menuitem, 12, trap_Cvar_Value( "r_portalmaps_maxtexsize" ) == 0 ? maxtexsize : trap_Cvar_Value( "r_portalmaps_maxtexsize" ), 1, maxtexsize );
	if( !trap_Cvar_Value( "r_portalmaps_maxtexsize" ) )
		menuitem->disabled = qtrue;
	else
		menuitem->disabled = qfalse;
	yoffset += trap_SCR_strHeight( menuitem->font );

	menuitem = UI_InitMenuItem( "m_performanceadv_shadows", "shadows", 0, yoffset, MTYPE_SPINCONTROL, ALIGN_RIGHT_TOP, uis.fontSystemSmall, ShadowsControl );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
#ifdef CGAMEGETLIGHTORIGIN
	UI_SetupSpinControl( menuitem, shadows_names, trap_Cvar_Value( "cg_shadows" ) );
#else
	UI_SetupSpinControl( menuitem, offon_names, trap_Cvar_Value( "r_shadows" ) );
#endif
	yoffset += trap_SCR_strHeight( menuitem->font );

	menuitem = UI_InitMenuItem( "m_performanceadv_shadowmap_maxtexsize", "shadow maps texture size", 0, yoffset, MTYPE_SPINCONTROL, ALIGN_RIGHT_TOP, uis.fontSystemSmall, ShadowMapMaxTexSizeControl );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	UI_SetupSpinControl( menuitem, maxtexsize_names, trap_Cvar_Value( "r_shadows_maxtexsize" ) );
	yoffset += trap_SCR_strHeight( menuitem->font );
	menuitem = UI_InitMenuItem( "m_performanceadv_shadowmap_maxtexsize_slider", "", 0, yoffset, MTYPE_SLIDER, ALIGN_RIGHT_TOP, uis.fontSystemSmall, NULL );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	UI_SetupSlider( menuitem, 12, trap_Cvar_Value( "r_shadows_maxtexsize" ) == 0 ? maxtexsize : trap_Cvar_Value( "r_shadows_maxtexsize" ), 1, maxtexsize );
	if( !trap_Cvar_Value( "r_shadows_maxtexsize" ) )
		menuitem->disabled = qtrue;
	else
		menuitem->disabled = qfalse;
	yoffset += trap_SCR_strHeight( menuitem->font );

	menuitem = UI_InitMenuItem( "m_performanceadv_shadowmapsPCF", "shadow maps smoothing factor (PCF)", 0, yoffset, MTYPE_SLIDER, ALIGN_RIGHT_TOP, uis.fontSystemSmall, NULL );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	UI_SetupSlider( menuitem, 12, trap_Cvar_Value( "r_shadows_pcf" ), 0, 2 );
	yoffset += trap_SCR_strHeight( menuitem->font );

	menuitem = UI_InitMenuItem( "m_performanceadv_outlineworld", "outline world", 0, yoffset, MTYPE_SPINCONTROL, ALIGN_RIGHT_TOP, uis.fontSystemSmall, NULL );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	UI_SetupSpinControl( menuitem, offon_names, trap_Cvar_Value( "cg_outlineWorld" ) );
	yoffset += trap_SCR_strHeight( menuitem->font );
	if( !trap_Cvar_Value( "gl_ext_GLSL" ) )
		menuitem->disabled = qtrue;
	else
		menuitem->disabled = qfalse;

	menuitem = UI_InitMenuItem( "m_performanceadv_outlinemodels", "outline models", 0, yoffset, MTYPE_SPINCONTROL, ALIGN_RIGHT_TOP, uis.fontSystemSmall, NULL );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	UI_SetupSpinControl( menuitem, offon_names, trap_Cvar_Value( "cg_outlineModels" ) );
	yoffset += trap_SCR_strHeight( menuitem->font );

#if 0
	menuitem = UI_InitMenuItem( "m_performanceadv_glsl", "GLSL extension", 0, yoffset, MTYPE_SPINCONTROL, ALIGN_RIGHT_TOP, uis.fontSystemSmall, NULL );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	UI_SetupSpinControl( menuitem, offon_names, trap_Cvar_Value( "gl_ext_GLSL" ) );
	yoffset += trap_SCR_strHeight( menuitem->font );
#endif

	menuitem = UI_InitMenuItem( "m_performanceadv_deluxemapping", "deluxemapping", 0, yoffset, MTYPE_SPINCONTROL, ALIGN_RIGHT_TOP, uis.fontSystemSmall, NULL );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	UI_SetupSpinControl( menuitem, offon_names, trap_Cvar_Value( "r_lighting_deluxemapping" ) );
	yoffset += trap_SCR_strHeight( menuitem->font );
	if( trap_Cvar_Value( "gl_ext_GLSL" ) )
		menuitem->disabled = qfalse;
	else
		menuitem->disabled = qtrue;

	menuitem = UI_InitMenuItem( "m_performanceadv_dynamiclight", "dynamic light", 0, yoffset, MTYPE_SPINCONTROL, ALIGN_RIGHT_TOP, uis.fontSystemSmall, NULL );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	UI_SetupSpinControl( menuitem, dynamiclights_names, trap_Cvar_Value( "r_dynamiclight" ) );
	yoffset += trap_SCR_strHeight( menuitem->font );

	menuitem = UI_InitMenuItem( "m_performanceadv_subdivisions", "curves geometric detail", 0, yoffset, MTYPE_SLIDER, ALIGN_RIGHT_TOP, uis.fontSystemSmall, NULL );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	UI_SetupSlider( menuitem, 12, 4-NEARESTEXPOF2( trap_Cvar_Value( "r_subdivisions" ) ), 0, 4 );
	yoffset += trap_SCR_strHeight( menuitem->font );

	menuitem = UI_InitMenuItem( "m_performanceadv_offsetmapping", "offsetmapping", 0, yoffset, MTYPE_SPINCONTROL, ALIGN_RIGHT_TOP, uis.fontSystemSmall, NULL );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	UI_SetupSpinControl( menuitem, offsetmapping_names, trap_Cvar_Value( "r_offsetmapping" ) );
	yoffset += trap_SCR_strHeight( menuitem->font );
	if( !trap_Cvar_Value( "gl_ext_GLSL" ) )
		menuitem->disabled = qtrue;
	else
		menuitem->disabled = qfalse;

	menuitem = UI_InitMenuItem( "m_performanceadv_bloom", "light bloom", 0, yoffset, MTYPE_SPINCONTROL, ALIGN_RIGHT_TOP, uis.fontSystemSmall, NULL );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	UI_SetupSpinControl( menuitem, offon_names, trap_Cvar_Value( "r_bloom" ) );
	yoffset += trap_SCR_strHeight( menuitem->font );

	yoffset += trap_SCR_strHeight( menuitem->font );

	menuitem = UI_InitMenuItem( "m_performanceadv_back", "back", -16, yoffset, MTYPE_ACTION, ALIGN_RIGHT_TOP, uis.fontSystemBig, M_genericBackFunc );
	Menu_AddItem( &s_performanceadv_menu, menuitem );
	menuitem = UI_InitMenuItem( "m_performanceadv_apply", "apply", 10, yoffset, MTYPE_ACTION, ALIGN_LEFT_TOP, uis.fontSystemBig, ApplyButton );
	Menu_AddItem( &s_performanceadv_menu, menuitem );

	Menu_Center( &s_performanceadv_menu );
	Menu_Init( &s_performanceadv_menu );
}

static void PerformanceAdv_MenuDraw( void )
{
	Menu_AdjustCursor( &s_performanceadv_menu, 1 );
	Menu_Draw( &s_performanceadv_menu );
}

static const char *PerformanceAdv_MenuKey( int key )
{
	return Default_MenuKey( &s_performanceadv_menu, key );
}

static const char *PerformanceAdv_MenuCharEvent( qwchar key )
{
	return Default_MenuCharEvent( &s_performanceadv_menu, key );
}

void M_Menu_PerformanceAdv_f( void )
{
	PerformanceAdv_Init();
	M_PushMenu( &s_performanceadv_menu, PerformanceAdv_MenuDraw, PerformanceAdv_MenuKey, PerformanceAdv_MenuCharEvent );
}
